class TrackEvent {
  TrackEvent({
    this.context,
    this.batch,
    this.sentAt,
  });

  Context context;
  List<Batch> batch;
  DateTime sentAt;

  factory TrackEvent.fromJson(Map<String, dynamic> json) => TrackEvent(
        context: Context.fromJson(json["context"]),
        batch: List<Batch>.from(json["batch"].map((x) => Batch.fromJson(x))),
        sentAt: DateTime.parse(json["sentAt"]),
      );

  Map<String, dynamic> toJson() => {
        "context": context.toJson(),
        "batch": List<dynamic>.from(batch.map((x) => x.toJson())),
        "sentAt": sentAt.toIso8601String(),
      };
}

class Batch {
  Batch({
    this.messageId,
    this.channel,
    this.anonymousId,
    this.userId,
    this.event,
    this.context,
    this.properties,
    this.type,
    this.timestamp,
    this.writeKey,
  });

  String messageId;
  String channel;
  String anonymousId;
  String userId;
  String event;
  Context context;
  Map<String, dynamic> properties;
  String type;
  DateTime timestamp;
  String writeKey;

  factory Batch.fromJson(Map<String, dynamic> json) => Batch(
        messageId: json["messageId"],
        channel: json["channel"],
        anonymousId: json["anonymousId"],
        userId: json["userId"],
        event: json["event"],
        context: Context.fromJson(json["context"]),
        properties: Map.from(json["properties"]),
        type: json["type"],
        timestamp: DateTime.parse(json["timestamp"]),
        writeKey: json["writeKey"],
      );

  Map<String, dynamic> toJson() => {
        "messageId": messageId,
        "channel": channel,
        "anonymousId": anonymousId,
        "userId": userId,
        "event": event,
        "context": context.toJson(),
        "properties": properties,
        "type": type,
        "timestamp": timestamp.toIso8601String(),
        "writeKey": writeKey,
      };
}

class Context {
  Context({
    this.screen,
    this.os,
    this.contextLibrary,
    this.app,
    this.locale,
    this.device,
    this.timezone,
    this.userAgent,
    this.ip,
  });

  Screen screen;
  Library os;
  Library contextLibrary;
  App app;
  String locale;
  Device device;
  String timezone;
  String userAgent;
  String ip;

  factory Context.fromJson(Map<String, dynamic> json) => Context(
        screen: Screen.fromJson(json["screen"]),
        os: Library.fromJson(json["os"]),
        contextLibrary: Library.fromJson(json["library"]),
        app: App.fromJson(json["app"]),
        locale: json["locale"],
        device: Device.fromJson(json["device"]),
        timezone: json["timezone"],
        userAgent: json["userAgent"],
        ip: json["ip"],
      );

  Map<String, dynamic> toJson() => {
        "screen": screen.toJson(),
        "os": os.toJson(),
        "library": contextLibrary.toJson(),
        "app": app.toJson(),
        "locale": locale,
        "device": device.toJson(),
        "timezone": timezone,
        "userAgent": userAgent,
        "ip": ip,
      };
}

class App {
  App({
    this.build,
    this.namespace,
    this.name,
    this.version,
  });

  String build;
  String namespace;
  String name;
  String version;

  factory App.fromJson(Map<String, dynamic> json) => App(
        build: json["build"],
        namespace: json["namespace"],
        name: json["name"],
        version: json["version"],
      );

  Map<String, dynamic> toJson() => {
        "build": build,
        "namespace": namespace,
        "name": name,
        "version": version,
      };
}

class Library {
  Library({
    this.name,
    this.version,
  });

  String name;
  String version;

  factory Library.fromJson(Map<String, dynamic> json) => Library(
        name: json["name"],
        version: json["version"],
      );

  Map<String, dynamic> toJson() => {
        "name": name,
        "version": version,
      };
}

class Device {
  Device({
    this.manufacturer,
    this.model,
    this.id,
  });

  String manufacturer;
  String model;
  String id;

  factory Device.fromJson(Map<String, dynamic> json) => Device(
        manufacturer: json["manufacturer"],
        model: json["model"],
        id: json["id"],
      );

  Map<String, dynamic> toJson() => {
        "manufacturer": manufacturer,
        "model": model,
        "id": id,
      };
}

class Screen {
  Screen({
    this.width,
    this.height,
  });

  int width;
  int height;

  factory Screen.fromJson(Map<String, dynamic> json) => Screen(
        width: json["width"],
        height: json["height"],
      );

  Map<String, dynamic> toJson() => {
        "width": width,
        "height": height,
      };
}

class Properties {
  Properties({
    this.name,
    this.googleAdId,
    this.platform,
    this.languageCode,
    this.type,
    this.countryCode,
    this.component,
    this.platformNew,
    this.label,
    this.businessUnit,
  });

  String name;
  String googleAdId;
  String platform;
  String languageCode;
  String countryCode;
  String component;
  String platformNew;
  String label;
  String businessUnit;
  String type;

  factory Properties.fromJson(Map<String, dynamic> json) => Properties(
        name: json["pageType"],
        googleAdId: json["google_ad_id"],
        platform: json["platform"],
        type: json["type"],
        component: json["Template_Name_New"],
        platformNew: json["Platform_New"],
        label: json["Template_Type_New"],
        businessUnit: json["businessUnit"],
      );

  Map<String, dynamic> toJson() {
    Map<String, dynamic> map = new Map();

    map["name"] = name;
    map["google_ad_id"] = googleAdId;
    map["platform"] = platform;
    map["languageCode"] = languageCode;
    map["countryCode"] = countryCode;
    if (component != null && component.isNotEmpty) map["component"] = component;
    if (type != null && type.isNotEmpty) map["type"] = type;
    if (label != null && label.isNotEmpty) map["label"] = label;
    map["businessUnit"] = businessUnit;

    return map;
  }
}
